package com.persagy.apm.alarmservice.dependency.rwd.service;

import com.alibaba.fastjson.JSONObject;
import com.persagy.apm.alarmservice.dependency.rwd.client.DmpRwdClient;
import com.persagy.apm.alarmservice.group.management.model.equip.dto.QueryEquipDTO;
import com.persagy.apm.alarmservice.group.management.model.equip.vo.EquipItem;
import com.persagy.apm.energyalarmstarter.alarmdata.feign.DmpResult;
import com.persagy.apm.energyalarmstarter.alarmdata.model.dto.common.UrlParamDTO;
import com.persagy.common.exception.BusinessException;
import org.apache.commons.lang3.StringUtils;
import org.assertj.core.util.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 报警中台物理世界service实现类
 *
 * @author lixing
 * @version V1.0 2021/10/20 2:26 下午
 **/
@Service
public class DmpRwdServiceImpl {
    @Autowired
    DmpRwdClient dmpRwdClient;

    /**
     * 获取物理世界对象详情信息
     *
     * @param urlParamDTO 物理世界通用查询参数
     * @param objId 对象id
     * @return 对象详情
     * @exception BusinessException 未查询到对象
     * @author lixing
     * @version V1.0 2021/10/20 3:17 下午
     */
    public EquipItem getObject(UrlParamDTO urlParamDTO, String objId) {
        if (StringUtils.isBlank(objId)) {
            return null;
        }
        List<JSONObject> records = queryRwdObjByIds(urlParamDTO, Lists.newArrayList(objId));
        JSONObject jsonObject = records.get(0);

        return convert2EquipItem(jsonObject);
    }

    /**
     * 根据对象id列表查询物理世界对象列表
     *
     * @param urlParamDTO 物理世界通用查询参数
     * @param objIdList 对象id列表
     * @return 物理世界对象列表
     * @exception
     * @author lixing
     * @version V1.0 2021/10/20 4:38 下午
     */
    private List<JSONObject> queryRwdObjByIds(UrlParamDTO urlParamDTO, List<String> objIdList) {
        JSONObject queryObject = new JSONObject();
        JSONObject criteria = new JSONObject();
        criteria.put("id", objIdList);
        queryObject.put("criteria", criteria);
        DmpResult<List<JSONObject>> pageListCommonResult = dmpRwdClient.objQuery(urlParamDTO, queryObject);
        if (pageListCommonResult.getData() == null) {
            throw new BusinessException("物理世界中未获取到要查询的对象，id:" + objIdList);
        }
        List<JSONObject> records = pageListCommonResult.getData();
        if (CollectionUtils.isEmpty(records)) {
            throw new BusinessException("物理世界中未获取到要查询的对象，id:" + objIdList);
        }
        return records;
    }

    /**
     * 物理世界的对象信息转换为EquipItem对象
     *
     * @param jsonObject 物理世界的对象信息
     * @return EquipItem对象
     * @author lixing
     * @version V1.0 2021/10/20 3:19 下午
     */
    private EquipItem convert2EquipItem(JSONObject jsonObject) {
        EquipItem equipItem = new EquipItem();
        equipItem.setId(jsonObject.getString("id"));
        equipItem.setClassCode(jsonObject.getString("classCode"));
        equipItem.setLocalName(jsonObject.getString("localName"));
        equipItem.setInfos(jsonObject.toJSONString());
        return equipItem;
    }

    /**
     * 获取物理世界对象详情信息
     *
     * @param urlParamDTO 物理世界通用查询参数
     * @param objIds 对象id列表
     * @return 物理世界对象详情列表
     * @author lixing
     * @version V1.0 2021/10/20 3:17 下午
     */
    public List<EquipItem> queryObjListByIds(UrlParamDTO urlParamDTO, List<String> objIds) {
        if (CollectionUtils.isEmpty(objIds)) {
            return new ArrayList<>();
        }
        List<JSONObject> records = queryRwdObjByIds(urlParamDTO, objIds);
        if (records == null) {
            return new ArrayList<>();
        }
        return records.stream().map(this::convert2EquipItem).collect(Collectors.toList());
    }

    /**
     * 获取物理世界对象详情信息
     *
     * @param urlParamDTO 物理世界通用查询参数
     * @param queryEquipDTO 查询条件
     * @return 物理世界对象详情列表
     * @author lixing
     * @version V1.0 2021/10/20 3:17 下午
     */
    public List<EquipItem> queryObjList(UrlParamDTO urlParamDTO, QueryEquipDTO queryEquipDTO) {
        JSONObject queryObject = new JSONObject();
        JSONObject criteria = new JSONObject();
        String condition = queryEquipDTO.getCondition();
        if (StringUtils.isNotBlank(condition)) {
            condition = "{" + condition + "}";
            criteria = JSONObject.parseObject(condition);
        }
        if (StringUtils.isNotBlank(queryEquipDTO.getClassCode())) {
            criteria.put("classCode", queryEquipDTO.getClassCode());
        }

        /* 设备名称模糊查询 */
        if (StringUtils.isNotBlank(queryEquipDTO.getLocalName())) {
            JSONObject localNameObj = new JSONObject();
            localNameObj.put("$like", "%"+queryEquipDTO.getLocalName()+"%");
            criteria.put("localName", localNameObj);
        }

        queryObject.put("criteria", criteria);
        DmpResult<List<JSONObject>> pageListCommonResult = dmpRwdClient.objQuery(urlParamDTO, queryObject);

        if (pageListCommonResult.getData() == null) {
            return new ArrayList<>();
        }
        List<JSONObject> records = pageListCommonResult.getData();
        if (CollectionUtils.isEmpty(records)) {
            return new ArrayList<>();
        }
        return records.stream().map(this::convert2EquipItem).collect(Collectors.toList());
    }
}
