package com.persagy.apm.alarmservice.group.alarmrecord.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.persagy.apm.alarmservice.group.alarmrecord.enums.*;
import com.persagy.apm.alarmservice.group.alarmrecord.model.vo.AlarmRecordItemVO;
import com.persagy.apm.alarmservice.group.alarmrecord.model.vo.AlarmRecordListItem;
import com.persagy.apm.alarmservice.group.management.model.equip.vo.EquipItem;
import com.persagy.apm.alarmservice.group.management.service.GroupManagementService;
import com.persagy.apm.alarmservice.project.alarmcondition.model.vo.AlarmConditionStrItem;
import com.persagy.apm.common.model.vo.SimpleObjVO;
import com.persagy.apm.energyalarmstarter.alarmdata.feign.DmpResult;
import com.persagy.apm.energyalarmstarter.alarmdata.model.dto.alarmitem.QueryAlarmItemDTO;
import com.persagy.apm.energyalarmstarter.alarmdata.model.dto.alarmrecord.QueryAlarmRecordDTO;
import com.persagy.apm.energyalarmstarter.alarmdata.model.vo.AlarmItem;
import com.persagy.apm.energyalarmstarter.alarmdata.model.vo.AlarmRecordItem;
import com.persagy.apm.energyalarmstarter.alarmdata.service.AlarmItemServiceImpl;
import com.persagy.apm.energyalarmstarter.alarmdata.service.AlarmRecordServiceImpl;
import com.persagy.apm.energyalarmstarter.alarmdata.utils.DateUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.FillPatternType;
import org.apache.poi.ss.usermodel.IndexedColors;
import org.apache.poi.xssf.usermodel.XSSFCellStyle;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.assertj.core.util.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 报警记录包装类，提供对报警相关字段的转换方法
 *
 * @author lixing
 * @version V1.0 2021/9/20 4:29 下午
 **/
@Service
public class AlarmRecordWrapper {
    @Autowired
    AlarmItemServiceImpl alarmItemService;
    @Autowired
    GroupManagementService groupManagementService;
    @Autowired
    AlarmRecordServiceImpl alarmRecordService;

    /**
     * 获取报警类型编码和名称的映射
     *
     * @param itemCodes 报警类型编码列表
     * @return 报警类型编码 -> simpleObjVO(id, name)
     * @throws
     * @author lixing
     * @version V1.0 2021/9/20 4:31 下午
     */
    public Map<String, SimpleObjVO> getAlarmItemMap(List<String> itemCodes) throws Exception {
        Map<String, SimpleObjVO> alarmItemMap = new HashMap<>();
        QueryAlarmItemDTO queryAlarmItemDTO = new QueryAlarmItemDTO();
        queryAlarmItemDTO.setCode(itemCodes);
        DmpResult<List<AlarmItem>> alarmItemsResult = alarmItemService.query(queryAlarmItemDTO);
        List<AlarmItem> alarmItems = alarmItemsResult.getData();
        if (CollectionUtils.isEmpty(alarmItems)) {

        }
        alarmItemMap.putAll(alarmItems.stream().collect(Collectors.toMap(
                AlarmItem::getId,
                alarmItem -> new SimpleObjVO(alarmItem.getId(), alarmItem.getName()),
                (k1, k2) -> k1
        )));
        return alarmItemMap;
    }

    /**
     * 获取两个时间的时间间隔（小时）
     *
     * @param triggerTime 触发时间
     * @param endTime     结束时间
     * @return 两个时间的时间间隔（小时）
     * @author lixing
     * @version V1.0 2021/9/20 4:45 下午
     */
    public static String getAlarmDuration(Date triggerTime, Date endTime) {
        if (endTime == null) {
            endTime = new Date();
        }
        if (triggerTime == null) {
            return null;
        }

        long durationNs = endTime.getTime() - triggerTime.getTime();
        // ns/1000=s s/60=min min/60=hour
        // 保留两位小数
        BigDecimal bd = new BigDecimal(durationNs);
        bd = bd.divide(new BigDecimal(1000 * 60 * 60),2, RoundingMode.HALF_UP);
        return bd.toString();
    }

    /**
     * 将报警数据中台的查询结果转换为报警服务中台的报警记录列表
     *
     * @param dmpResult 报警数据中台的查询结果
     * @return 报警服务中台的报警记录列表
     * @author lixing
     * @version V1.0 2021/9/20 6:16 下午
     */
    public List<AlarmRecordListItem> getAlarmRecordListItems(DmpResult<List<AlarmRecordItem>> dmpResult) throws Exception {
        // 获取查询结果中的设备
        Set<String> objIds = dmpResult.getData().stream().map(AlarmRecordItem::getObjId).collect(Collectors.toSet());
        List<String> objIdList = Lists.newArrayList(objIds);
        // 获取这些设备所在的分组信息
        Map<String, SimpleObjVO> objGroupMap = groupManagementService.queryObjGroup(objIdList);
        // 获取这些设备的名称
        Map<String, EquipItem> objMap = groupManagementService.queryEquipsByIds(objIdList);
        if (CollectionUtils.isEmpty(dmpResult.getData())) {
            return new ArrayList<>();
        }
        // 获取查询结果中的报警类型
        List<String> itemCodes = dmpResult.getData().stream().map(AlarmRecordItem::getItemCode).
                collect(Collectors.toList());
        Map<String, SimpleObjVO> alarmItemMap = getAlarmItemMap(itemCodes);

        return dmpResult.getData().stream().map(alarmRecordItem ->
                getAlarmRecordListItem(objGroupMap, objMap, alarmItemMap, alarmRecordItem)).
                collect(Collectors.toList());
    }

    /**
     * 将报警数据中台返回的一条记录转换为报警服务中台报警记录列表中的一条数据
     *
     * @param objGroupMap     报警对象id和分组的映射关系
     * @param objMap          报警对象id和报警对象的映射关系
     * @param alarmItemMap    报警类型编码和报警类型的映射关系
     * @param alarmRecordItem 报警数据中台返回的一条报警记录
     * @return 报警服务中台报警记录列表中的一条数据
     * @author lixing
     * @version V1.0 2021/9/20 6:21 下午
     */
    private AlarmRecordListItem getAlarmRecordListItem(
            Map<String, SimpleObjVO> objGroupMap,
            Map<String, EquipItem> objMap,
            Map<String, SimpleObjVO> alarmItemMap,
            AlarmRecordItem alarmRecordItem) {
        AlarmRecordListItem alarmRecordListItem = new AlarmRecordListItem();
        alarmRecordListItem.setId(alarmRecordItem.getId());
        alarmRecordListItem.setGroup(objGroupMap.get(alarmRecordItem.getObjId()));
        EquipItem objInfo = objMap.get(alarmRecordItem.getObjId());
        alarmRecordListItem.setObj(new SimpleObjVO(alarmRecordItem.getObjId(),
                objInfo == null ? null : objInfo.getLocalName()));
        alarmRecordListItem.setAlarmItem(alarmItemMap.get(alarmRecordItem.getItemCode()));
        alarmRecordListItem.setTriggerTime(alarmRecordItem.getTriggerTime());
        // 计算报警的持续时长
        alarmRecordListItem.setDuration(
                AlarmRecordWrapper.getAlarmDuration(
                        alarmRecordItem.getTriggerTime(),
                        alarmRecordItem.getEndTime()
                )
        );
        EnumAlarmLevel alarmLevel = EnumAlarmLevel.getByType(alarmRecordItem.getLevel());
        if (alarmLevel != null) {
            alarmRecordListItem.setAlarmLevel(new SimpleObjVO(
                    alarmLevel.getType(), alarmLevel.getDesc()
            ));
        }

        EnumAlarmState alarmState = EnumAlarmState.getByType(
                String.valueOf(alarmRecordItem.getState()));
        if (alarmState != null) {
            alarmRecordListItem.setAlarmState(new SimpleObjVO(
                    alarmState.getType(), alarmState.getDesc()
            ));
        }

        EnumAlarmTreatState alarmTreatState = EnumAlarmTreatState.getByType(
                String.valueOf(alarmRecordItem.getTreatState()));
        if (alarmTreatState != null) {
            alarmRecordListItem.setAlarmTreatState(new SimpleObjVO(
                    alarmTreatState.getType(), alarmTreatState.getDesc()
            ));
        }

        EnumAlarmTreatMode alarmTreatMode = EnumAlarmTreatMode.getByType(
                String.valueOf(alarmRecordItem.getTreatMode()));
        if (alarmTreatMode != null) {
            alarmRecordListItem.setAlarmTreatMode(new SimpleObjVO(
                    alarmTreatMode.getType(), alarmTreatMode.getDesc()
            ));
        }

        alarmRecordListItem.setIgnoreTime(alarmRecordItem.getIgnoreTime());
        alarmRecordListItem.setTreatEndTime(alarmRecordItem.getTreatEndTime());
        //报警描述
        alarmRecordListItem.setRemark(alarmRecordItem.getRemark());
        //补充说明
        alarmRecordListItem.setSupplement(alarmRecordItem.getSupplement());
        alarmRecordListItem.setCategory(alarmRecordItem.getCategory());
        return alarmRecordListItem;
    }

    /**
     * 查看报警记录详情
     *
     * @param recordId 报警记录id
     * @return 报警记录详情
     * @author lixing
     * @version V1.0 2021/9/21 3:54 下午
     */
    public AlarmRecordItem getAlarmRecord(String recordId) throws Exception {
        QueryAlarmRecordDTO queryAlarmRecordDTO = new QueryAlarmRecordDTO();
        queryAlarmRecordDTO.setId(Lists.newArrayList(recordId));
        DmpResult<List<AlarmRecordItem>> result = alarmRecordService.query(queryAlarmRecordDTO);
        if (result == null) {
            return null;
        }
        List<AlarmRecordItem> data = result.getData();
        if (CollectionUtils.isEmpty(data)) {
            return null;
        }
        return data.get(0);
    }

    /**
     * 将报警数据中台的报警记录封装为业务中台的报警记录对象
     *
     * @param alarmRecordItem 数据中台的报警记录
     * @return 业务中台的报警记录对象
     * @author lixing
     * @version V1.0 2021/9/21 5:44 下午
     */
    public AlarmRecordItemVO convert2AlarmRecordItemVO(AlarmRecordItem alarmRecordItem) throws Exception {
        if (alarmRecordItem == null) {
            return null;
        }
        AlarmRecordItemVO alarmRecordItemVO = new AlarmRecordItemVO();
        /* 报警基本信息 */
        copyBasicInfo(alarmRecordItem, alarmRecordItemVO);
        /* 报警条件 */
        JSONObject condition = alarmRecordItem.getCondition();
        if (condition != null) {
            AlarmConditionStrItem alarmConditionStrItem = new AlarmConditionStrItem();
            // 报警规则
            JSONArray array = condition.getJSONArray("rules");
            alarmConditionStrItem.setRules(array.toJavaList(String.class));
            // 报警触发时间转换为分钟
            Integer triggerUphold = condition.getInteger("triggerUphold");
            if (triggerUphold != null) {
                triggerUphold = triggerUphold / 60;
            }
            String lastTimeInfo = "持续%s分钟满足报警触发规则，则产生报警";
            alarmConditionStrItem.setLastTimeInfo(
                    String.format(lastTimeInfo, triggerUphold == null ? "--" : triggerUphold));
            EnumAlarmLevel alarmLevel = EnumAlarmLevel.getByType(alarmRecordItem.getLevel());
            if (alarmLevel != null) {
                alarmConditionStrItem.setAlarmLevel(new SimpleObjVO(
                        alarmLevel.getType(), alarmLevel.getDesc()
                ));
            }
            alarmRecordItemVO.setAlarmCondition(alarmConditionStrItem);
        }

        alarmRecordItemVO.setTreatEndTime(alarmRecordItem.getTreatEndTime());
        alarmRecordItemVO.setIgnoreTime(alarmRecordItem.getIgnoreTime());
        alarmRecordItemVO.setRemark(alarmRecordItem.getRemark());
        alarmRecordItemVO.setSupplement(alarmRecordItem.getSupplement());
        alarmRecordItemVO.setCategory(alarmRecordItem.getCategory());

        return alarmRecordItemVO;
    }

    /**
     * 将报警数据中台报警对象的基本信息复制到报警业务中台报警对象
     *
     * @param alarmRecordItem   报警数据中台报警对象
     * @param alarmRecordItemVO 报警业务中台报警对象
     * @author lixing
     * @version V1.0 2021/9/22 3:41 下午
     */
    public void copyBasicInfo(AlarmRecordItem alarmRecordItem, AlarmRecordItemVO alarmRecordItemVO) throws Exception {
        alarmRecordItemVO.setId(alarmRecordItem.getId());
        Map<String, SimpleObjVO> objGroupMap = groupManagementService.queryObjGroup(
                Lists.newArrayList(alarmRecordItem.getObjId()));
        alarmRecordItemVO.setGroup(objGroupMap.get(alarmRecordItem.getObjId()));
        // 获取这些设备的名称
        Map<String, EquipItem> objMap = groupManagementService.queryEquipsByIds(
                Lists.newArrayList(alarmRecordItem.getObjId())
        );
        EquipItem objInfo = objMap.get(alarmRecordItem.getObjId());
        alarmRecordItemVO.setObj(new SimpleObjVO(alarmRecordItem.getObjId(),
                objInfo == null ? null : objInfo.getLocalName()));
        // 报警类型
        Map<String, SimpleObjVO> alarmItemMap = getAlarmItemMap(Lists.newArrayList(alarmRecordItem.getItemCode()));
        SimpleObjVO simpleObjVO = alarmItemMap.get(alarmRecordItem.getItemCode());
        alarmRecordItemVO.setAlarmItem(simpleObjVO);

        alarmRecordItemVO.setTriggerTime(alarmRecordItem.getTriggerTime());
        alarmRecordItemVO.setEndTime(alarmRecordItem.getEndTime());
        alarmRecordItemVO.setDuration(getAlarmDuration(alarmRecordItem.getTriggerTime(), alarmRecordItem.getEndTime()));

        EnumAlarmLevel alarmLevel = EnumAlarmLevel.getByType(alarmRecordItem.getLevel());
        if (alarmLevel != null) {
            alarmRecordItemVO.setAlarmLevel(new SimpleObjVO(
                    alarmLevel.getType(), alarmLevel.getDesc()
            ));
        }

        EnumAlarmState alarmState = EnumAlarmState.getByType(
                String.valueOf(alarmRecordItem.getState()));
        if (alarmState != null) {
            alarmRecordItemVO.setAlarmState(new SimpleObjVO(
                    alarmState.getType(), alarmState.getDesc()
            ));
        }

        EnumAlarmTreatState alarmTreatState = EnumAlarmTreatState.getByType(
                String.valueOf(alarmRecordItem.getTreatState()));
        if (alarmTreatState != null) {
            alarmRecordItemVO.setAlarmTreatState(new SimpleObjVO(
                    alarmTreatState.getType(), alarmTreatState.getDesc()
            ));
        }

        EnumAlarmTreatMode alarmTreatMode = EnumAlarmTreatMode.getByType(
                String.valueOf(alarmRecordItem.getTreatMode()));
        if (alarmTreatMode != null) {
            alarmRecordItemVO.setAlarmTreatMode(new SimpleObjVO(
                    alarmTreatMode.getType(), alarmTreatMode.getDesc()
            ));
        }
        /* 近30天报警触发次数 */
        Integer count = getAlarmCountInMonth(
                alarmRecordItem.getTriggerTime(), alarmRecordItem.getItemCode(), alarmRecordItem.getObjId());
        alarmRecordItemVO.setAlarmCountInMonth(count);
    }

    /**
     * 指定报警对象报警类型在指定日期前一个月内发生报警的次数
     *
     * @param date     指定日期
     * @param itemCode 指定报警类型
     * @param objId    指定报警对象
     * @return 发生报警的次数
     * @author lixing
     * @version V1.0 2021/9/22 3:43 下午
     */
    public Integer getAlarmCountInMonth(Date date, String itemCode, String objId) throws Exception {
        if (date == null || StringUtils.isBlank(itemCode)) {
            return 0;
        }
        QueryAlarmRecordDTO queryAlarmRecordDTO = new QueryAlarmRecordDTO();
        queryAlarmRecordDTO.setTriggerDurationStart(
                DateUtils.getMonthOff(date, -1));
        queryAlarmRecordDTO.setTriggerDurationEnd(date);
        queryAlarmRecordDTO.setItemCode(Lists.newArrayList(itemCode));
        queryAlarmRecordDTO.setObjId(Lists.newArrayList(objId));
        return alarmRecordService.queryCount(queryAlarmRecordDTO);
    }

    /**
     * 查看报警记录详情
     *
     * @param recordIdList 报警记录id列表
     * @return 报警记录详情列表
     * @author lixing
     * @version V1.0 2021/9/21 3:54 下午
     */
    public List<AlarmRecordItem> getAlarmRecordList(List<String> recordIdList) throws Exception {
        if (CollectionUtils.isEmpty(recordIdList) || recordIdList.size() == 0) {
            return null;
        }
        QueryAlarmRecordDTO queryAlarmRecordDTO = new QueryAlarmRecordDTO();
        queryAlarmRecordDTO.setId(recordIdList);
        DmpResult<List<AlarmRecordItem>> result = alarmRecordService.query(queryAlarmRecordDTO);
        if (result == null) {
            return null;
        }
        return result.getData();
    }

    /**
     * 填充 excel文件数据行
     *
     * @param enumDownloadAlarmListType     报警列表类型
     * @param sheet                 文件sheet
     * @param alarmRecordItemVOList 数据列表
     */
    public void fillDataTable(EnumDownloadAlarmListType enumDownloadAlarmListType, XSSFSheet sheet, List<AlarmRecordItemVO> alarmRecordItemVOList) {
        try {
            for (int i = 0; i < alarmRecordItemVOList.size(); i++) {
                AlarmRecordItemVO alarmRecordItemVO = alarmRecordItemVOList.get(i);
                XSSFRow row = sheet.createRow(i + 1);
                String groupName = "";//分组名称
                if (null != alarmRecordItemVO.getGroup() && null != alarmRecordItemVO.getGroup().getName()) {
                    groupName = alarmRecordItemVO.getGroup().getName();
                }
                Cell cell = row.createCell(0);
                cell.setCellValue(groupName);
                String objName = "";//对象名称
                if (null != alarmRecordItemVO.getObj() && null != alarmRecordItemVO.getObj().getName()) {
                    objName = alarmRecordItemVO.getObj().getName();
                }
                cell = row.createCell(1);
                cell.setCellValue(objName);
                String itemName = "";//报警类型
                if (null != alarmRecordItemVO.getAlarmItem() && null != alarmRecordItemVO.getAlarmItem().getName()) {
                    itemName = alarmRecordItemVO.getAlarmItem().getName();
                }
                cell = row.createCell(2);
                cell.setCellValue(itemName);
                String levelName = "";//报警级别
                if (null != alarmRecordItemVO.getAlarmLevel() && null != alarmRecordItemVO.getAlarmLevel().getName()) {
                    levelName = alarmRecordItemVO.getAlarmLevel().getName();
                }
                String stateName = "";//报警状态
                if (null != alarmRecordItemVO.getAlarmState() && null != alarmRecordItemVO.getAlarmState().getName()) {
                    stateName = alarmRecordItemVO.getAlarmState().getName();
                }
                String treatStateName = "";//报警处理状态
                if (null != alarmRecordItemVO.getAlarmTreatState() && null != alarmRecordItemVO.getAlarmTreatState().getName()) {
                    treatStateName = alarmRecordItemVO.getAlarmTreatState().getName();
                }
                String triggerTime = "";
                if (null != alarmRecordItemVO.getTriggerTime()) {
                    triggerTime = DateUtils.date2Str(alarmRecordItemVO.getTriggerTime(), DateUtils.SDF_SECOND);
                }
                String treatEndTime = "";
                if (null != alarmRecordItemVO.getTreatEndTime()) {
                    treatEndTime = DateUtils.date2Str(alarmRecordItemVO.getTreatEndTime(), DateUtils.SDF_SECOND);
                }
                String ignoreTime = "";
                if (null != alarmRecordItemVO.getIgnoreTime()) {
                    ignoreTime = DateUtils.date2Str(alarmRecordItemVO.getIgnoreTime(), DateUtils.SDF_SECOND);
                }
                String duration = "";
                if (null != alarmRecordItemVO.getDuration()) {
                    duration = alarmRecordItemVO.getDuration();
                }
                cell = row.createCell(3);
                cell.setCellValue(triggerTime);
                if (EnumDownloadAlarmListType.CURRENT.getType().equals(enumDownloadAlarmListType.getType())) {
                    cell = row.createCell(4);
                    cell.setCellValue(duration);
                    cell = row.createCell(5);
                    cell.setCellValue(levelName);
                    cell = row.createCell(6);
                    cell.setCellValue(stateName);
                    cell = row.createCell(7);
                    cell.setCellValue(treatStateName);
                } else if (EnumDownloadAlarmListType.HISTORY.getType().equals(enumDownloadAlarmListType.getType())) {
                    cell = row.createCell(4);
                    cell.setCellValue(treatEndTime);
                    cell = row.createCell(5);
                    cell.setCellValue(duration);
                    cell = row.createCell(6);
                    cell.setCellValue(levelName);
                    cell = row.createCell(7);
                    cell.setCellValue(stateName);
                    cell = row.createCell(8);
                    cell.setCellValue(treatStateName);
                } else if (EnumDownloadAlarmListType.IGNORE.getType().equals(enumDownloadAlarmListType.getType())) {
                    cell = row.createCell(4);
                    cell.setCellValue(ignoreTime);
                    cell = row.createCell(5);
                    cell.setCellValue(duration);
                    cell = row.createCell(6);
                    cell.setCellValue(levelName);
                    cell = row.createCell(7);
                    cell.setCellValue(stateName);
                    cell = row.createCell(8);
                    cell.setCellValue(treatStateName);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    /**
     * 处理文件标题行
     *
     * @param enumDownloadAlarmListType 报警列表类型
     * @param sheet             文件sheet
     * @param wb                文件wb
     */
    public void handleExcelTitle(EnumDownloadAlarmListType enumDownloadAlarmListType, XSSFSheet sheet, XSSFWorkbook wb) {
        XSSFCellStyle cellStyle = wb.createCellStyle();
        cellStyle.setWrapText(true);
        cellStyle.setFillForegroundColor(IndexedColors.GREY_50_PERCENT.getIndex());
        cellStyle.setFillPattern(FillPatternType.SOLID_FOREGROUND);
        //设置行宽
        sheet.setColumnWidth(0, 5000);
        sheet.setColumnWidth(1, 5000);
        sheet.setColumnWidth(2, 7000);
        sheet.setColumnWidth(3, 5000);
        //行
        XSSFRow Row1 = sheet.createRow(0);
        Cell cell = Row1.createCell(0);
        cell.setCellStyle(cellStyle);
        cell.setCellValue("分组名称");
        cell = Row1.createCell(1);
        cell.setCellStyle(cellStyle);
        cell.setCellValue("报警对象");
        cell = Row1.createCell(2);
        cell.setCellStyle(cellStyle);
        cell.setCellValue("报警类型");
        cell = Row1.createCell(3);
        cell.setCellStyle(cellStyle);
        cell.setCellValue("报警开始时间");
        if (EnumDownloadAlarmListType.CURRENT.getType().equals(enumDownloadAlarmListType.getType())) {
            sheet.setColumnWidth(4, 4500);
            sheet.setColumnWidth(5, 3000);
            sheet.setColumnWidth(6, 3000);
            sheet.setColumnWidth(7, 5000);
            cell = Row1.createCell(4);
            cell.setCellStyle(cellStyle);
            cell.setCellValue("报警持续时长(h)");
            cell = Row1.createCell(5);
            cell.setCellStyle(cellStyle);
            cell.setCellValue("报警等级");
            cell = Row1.createCell(6);
            cell.setCellStyle(cellStyle);
            cell.setCellValue("报警状态");
            cell = Row1.createCell(7);
            cell.setCellStyle(cellStyle);
            cell.setCellValue("报警处理状态");
        } else if (EnumDownloadAlarmListType.HISTORY.getType().equals(enumDownloadAlarmListType.getType())) {
            sheet.setColumnWidth(4, 5000);
            sheet.setColumnWidth(5, 3500);
            sheet.setColumnWidth(6, 3000);
            sheet.setColumnWidth(7, 3000);
            sheet.setColumnWidth(8, 5000);
            cell = Row1.createCell(4);
            cell.setCellStyle(cellStyle);
            cell.setCellValue("处理完成时间");
            cell = Row1.createCell(5);
            cell.setCellStyle(cellStyle);
            cell.setCellValue("异常持续时长(h)");
            cell = Row1.createCell(6);
            cell.setCellStyle(cellStyle);
            cell.setCellValue("报警等级");
            cell = Row1.createCell(7);
            cell.setCellStyle(cellStyle);
            cell.setCellValue("报警状态");
            cell = Row1.createCell(8);
            cell.setCellStyle(cellStyle);
            cell.setCellValue("报警处理状态");
        } else if (EnumDownloadAlarmListType.IGNORE.getType().equals(enumDownloadAlarmListType.getType())) {
            sheet.setColumnWidth(4, 5000);
            sheet.setColumnWidth(5, 3500);
            sheet.setColumnWidth(6, 3000);
            sheet.setColumnWidth(7, 3000);
            sheet.setColumnWidth(8, 5000);
            cell = Row1.createCell(4);
            cell.setCellStyle(cellStyle);
            cell.setCellValue("忽略时间");
            cell = Row1.createCell(5);
            cell.setCellStyle(cellStyle);
            cell.setCellValue("异常持续时长(h)");
            cell = Row1.createCell(6);
            cell.setCellStyle(cellStyle);
            cell.setCellValue("报警等级");
            cell = Row1.createCell(7);
            cell.setCellStyle(cellStyle);
            cell.setCellValue("报警状态");
            cell = Row1.createCell(8);
            cell.setCellStyle(cellStyle);
            cell.setCellValue("报警处理状态");
        }
    }

}
