package com.persagy.apm.alarmservice.group.management.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import com.persagy.apm.alarmservice.dependency.rwd.service.DmpRwdServiceImpl;
import com.persagy.apm.alarmservice.group.management.model.equip.dto.QueryEquipDTO;
import com.persagy.apm.alarmservice.group.management.model.equip.vo.EquipItem;
import com.persagy.apm.alarmservice.group.management.model.group.ConvertGroupTool;
import com.persagy.apm.alarmservice.group.management.model.group.vo.EquipGroupInfoVO;
import com.persagy.apm.alarmservice.group.management.model.group.vo.GroupInfoVO;
import com.persagy.apm.alarmservice.group.management.model.group.vo.GroupTreeVO;
import com.persagy.apm.alarmservice.group.management.service.GroupManagementService;
import com.persagy.apm.alarmservice.group.objgroup.model.ObjGroup;
import com.persagy.apm.alarmservice.group.objgroup.model.dto.QueryObjGroupDTO;
import com.persagy.apm.alarmservice.group.objgroup.service.IObjGroupService;
import com.persagy.apm.common.context.poems.PoemsContext;
import com.persagy.apm.common.model.vo.SimpleObjVO;
import com.persagy.apm.energyalarmstarter.alarmdata.model.dto.common.UrlParamDTO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 分组管理service
 *
 * @author 易涛
 * @version 1.0
 * @date 2021/9/8 14:18
 */
@Slf4j
@Service
public class GroupManagementServiceImpl implements GroupManagementService {
    @Autowired
    private DmpRwdServiceImpl dmpRwdService;
    @Autowired
    private IObjGroupService objGroupService;

    @Override
    public EquipItem equipGet(String objId, String projectId, String groupCode) {
        UrlParamDTO urlParamDTO = new UrlParamDTO();
        urlParamDTO.setGroupCode(groupCode);
        urlParamDTO.setProjectId(projectId);
        return dmpRwdService.getObject(urlParamDTO, objId);
    }

    /**
     * 查询设备
     *
     * @param objIds 设备id列表
     * @return 设备列表
     * @author lixing
     * @version V1.0 2021/9/20 1:15 下午
     */
    private List<EquipItem> findEquipsByIds(List<String> objIds) {
        if (CollectionUtils.isEmpty(objIds)) {
            throw new IllegalArgumentException("批量查询传入的ids参数为空");
        }
        if (objIds.size() > 100) {
            throw new IllegalArgumentException("批量查询的参数过长（最长为100），参数ids的size为：" + objIds.size());
        }
        UrlParamDTO urlParamDTO = UrlParamDTO.builder().
                groupCode(PoemsContext.getContext().getGroupCode()).
                projectId(PoemsContext.getContext().getProjectId()).build();
        return dmpRwdService.queryObjListByIds(urlParamDTO, objIds);
    }

    @Override
    public Map<String, EquipItem> queryEquipsByIds(List<String> ids) {
        if (CollectionUtils.isEmpty(ids)) {
            return new HashMap<>();
        }
        Map<String, EquipItem> resultMap = new HashMap<>(ids.size());
        List<List<String>> partitions = Lists.partition(ids, 100);
        for (List<String> partition : partitions) {
            List<EquipItem> equipsByIds = findEquipsByIds(partition);
            if (CollectionUtils.isEmpty(equipsByIds)) {
                continue;
            }
            Map<String, EquipItem> partitionMap = equipsByIds.stream().collect(Collectors.toMap(
                    EquipItem::getId, equip -> equip, (k1, k2) -> k1
            ));
            resultMap.putAll(partitionMap);
        }
        return resultMap;
    }

    @Override
    public List<EquipGroupInfoVO> equipGroupQuery(List<String> ids) {
        List<EquipGroupInfoVO> equipGroupInfoVOList = Lists.newArrayList();
        List<EquipItem> equipItemList = findEquipsByIds(ids);
        for (EquipItem equipItem : equipItemList) {
            if (StringUtils.isEmpty(equipItem.getInfos())) {
                return equipGroupInfoVOList;
            }
            JSONObject jsonObject = JSONObject.parseObject(equipItem.getInfos());
            Map<String, String> equipFuncMap = new HashMap<>();
            if (jsonObject != null) {
                for (String key : jsonObject.keySet()) {
                    Object value = jsonObject.get(key);
                    if (value instanceof String) {
                        value = "\"" + value + "\"";
                    }
                    String funcInfo = key + ":" + value;
                    equipFuncMap.put(funcInfo, funcInfo);
                }
            }
            //获取分组树
            List<GroupTreeVO> groupTreeVOList = getGroupTree();
            EquipGroupInfoVO equipGroupInfoVO = new EquipGroupInfoVO();
            equipGroupInfoVO.setEquipId(equipItem.getId());
            equipGroupInfoVO.setClassCode(equipItem.getClassCode());
            if (CollectionUtils.isEmpty(groupTreeVOList)) {
                return equipGroupInfoVOList;
            }
            equipGroupInfoVOList.add(cascadeSearchGroupInfo(groupTreeVOList, equipGroupInfoVO, equipFuncMap));
        }
        return equipGroupInfoVOList;
    }

    @Override
    public Map<String, SimpleObjVO> queryObjGroup(List<String> objIds) {
        if (CollectionUtils.isEmpty(objIds)) {
            return new HashMap<>();
        }
        Map<String, SimpleObjVO> resultMap = new HashMap<>(objIds.size());

        // 每次查询100条
        List<List<String>> partitions = Lists.partition(objIds, 100);
        for (List<String> partition : partitions) {
            List<EquipGroupInfoVO> groupTreeInfoList = equipGroupQuery(partition);
            if (CollectionUtils.isEmpty(groupTreeInfoList)) {
                continue;
            }
            Map<String, SimpleObjVO> partitionMap = groupTreeInfoList.stream().collect(Collectors.toMap(
                    EquipGroupInfoVO::getEquipId,
                    equipGroupInfoVO -> new SimpleObjVO(equipGroupInfoVO.getId(), equipGroupInfoVO.getName()),
                    (k1, k2) -> k1
            ));
            resultMap.putAll(partitionMap);
        }

        return resultMap;
    }

    /**
     * 递归查询设备的分组信息
     * 遍历分组树，如果设备的信息点中包含分组的分组条件，同时设备的设备类编码与分组设备类编码匹配时，代表设备属于这个分组
     *
     * @param groupTreeVOList  分组树节点列表
     * @param equipGroupInfoVO 设备和所在分组信息(初始化时包含设备id和设备类编码)
     * @param equipFuncMap     "设备信息点:信息点取值" -> "设备信息点:信息点取值"，
     * @return 设备和所在分组信息（补充完成分组信息）
     * @author 易涛
     * @date 2021/9/10 11:24
     */
    private EquipGroupInfoVO cascadeSearchGroupInfo(
            List<GroupTreeVO> groupTreeVOList, EquipGroupInfoVO equipGroupInfoVO,
            Map<String, String> equipFuncMap) {
        if (StringUtils.isBlank(equipGroupInfoVO.getClassCode())) {
            throw new IllegalArgumentException("要查询的设备没有设备类信息");
        }
        for (GroupTreeVO groupTreeVO : groupTreeVOList) {
            if (!equipGroupInfoVO.getClassCode().equals(groupTreeVO.getClassCode())) {
                continue;
            }
            // 分组条件 排除掉配置的时候存在不小心放入空格的情况
            String groupCondition = groupTreeVO.getCondition().replace(" ", "");
            // 分组条件为空或设备信息点中包含分组条件
            if (StringUtils.isBlank(groupCondition) || equipFuncMap.containsKey(groupCondition)) {
                BeanUtils.copyProperties(groupTreeVO, equipGroupInfoVO);
                return equipGroupInfoVO;
            }

            if (!CollectionUtils.isEmpty(groupTreeVO.getChildren())) {
                cascadeSearchGroupInfo(groupTreeVO.getChildren(), equipGroupInfoVO, equipFuncMap);
            }
        }
        return equipGroupInfoVO;
    }

    /**
     * 获取分组树
     *
     * @return 分组树节点列表
     * @author lixing
     * @version V1.0 2021/10/19 2:40 下午
     */
    @Override
    public List<GroupTreeVO> getGroupTree() {
        // 从表中查询所有分组
        QueryObjGroupDTO queryObjGroupDTO = new QueryObjGroupDTO();
        List<ObjGroup> objGroups = objGroupService.queryObjGroupList(queryObjGroupDTO);

        if (CollectionUtils.isEmpty(objGroups)) {
            return new ArrayList<>();
        }
        // 将分组列表转换为树形结构，根节点的parentId = 0;
        Map<String, List<ObjGroup>> objGroupMap = new HashMap<>();
        for (ObjGroup objGroup : objGroups) {
            String parentId = objGroup.getParentId();
            if (!objGroupMap.containsKey(parentId)) {
                objGroupMap.put(parentId, Lists.newArrayList(objGroup));
            } else {
                objGroupMap.get(parentId).add(objGroup);
            }
        }
        return objGroupMap2Tree(objGroupMap);
    }

    /**
     * 将分组的map转换为树
     *
     * @param objGroupMap 分组的map
     * @return 分组树节点列表
     * @author lixing
     * @version V1.0 2021/10/19 2:50 下午
     */
    private List<GroupTreeVO> objGroupMap2Tree(Map<String, List<ObjGroup>> objGroupMap) {
        if (CollectionUtils.isEmpty(objGroupMap)) {
            return new ArrayList<>();
        }
        List<GroupTreeVO> result = new ArrayList<>();
        // 根节点parentId = 0;
        List<ObjGroup> rootObjGroups = objGroupMap.get("0");
        rootObjGroups.forEach(objGroup -> {
            GroupTreeVO groupTreeVO = ConvertGroupTool.INSTANCE.convert2GroupTreeVO(objGroup);
            result.add(groupTreeVO);
        });
        cascadeSetChildren(objGroupMap, result);
        return result;
    }

    /**
     * 级联设置子节点
     *
     * @param objGroupMap 分组map 分组parentId -> 分组列表
     * @param groupTreeVOList 要设置子节点的节点列表
     * @exception
     * @author lixing
     * @version V1.0 2021/10/19 3:16 下午
     */
    private void cascadeSetChildren(Map<String, List<ObjGroup>> objGroupMap, List<GroupTreeVO> groupTreeVOList) {
        groupTreeVOList.forEach(groupTreeVO -> {
            if (objGroupMap.containsKey(groupTreeVO.getId())) {
                List<ObjGroup> sonObjGroups = objGroupMap.get(groupTreeVO.getId());
                List<GroupTreeVO> sonNodes = sonObjGroups.stream().map(
                        ConvertGroupTool.INSTANCE::convert2GroupTreeVO
                ).collect(Collectors.toList());
                groupTreeVO.setChildren(sonNodes);
                cascadeSetChildren(objGroupMap, sonNodes);
            }
        });
    }

    @Override
    public List<EquipItem> getEquipList(String condition, String classCode) {
        QueryEquipDTO managementDTO = new QueryEquipDTO();
        managementDTO.setClassCode(classCode);
        managementDTO.setCondition(condition);
        return getEquipList(managementDTO);
    }

    @Override
    public List<EquipItem> getEquipList(QueryEquipDTO queryEquipDTO) {
        // 优先使用查询条件中的项目id，如果条件中项目id为空，使用线程中缓存的projectId
        String projectId = queryEquipDTO.getProjectId();
        if (StringUtils.isBlank(projectId)) {
            projectId = PoemsContext.getContext().getProjectId();
        }
        UrlParamDTO urlParamDTO = UrlParamDTO.builder().
                groupCode(PoemsContext.getContext().getGroupCode()).
                projectId(projectId).build();
        return dmpRwdService.queryObjList(urlParamDTO, queryEquipDTO);
    }

    @Override
    public GroupInfoVO getGroup(String id) {
        return queryById(getGroupTree(), new GroupInfoVO(), id);
    }

    /**
     * 递归匹配符合条件的分组对象
     *
     * @param groupTreeVOList
     * @param id
     * @return com.persagy.apm.alarmservice.group.management.model.vo.GroupTreeVO
     * @author 易涛
     * @date 2021/9/8 14:50
     */
    private GroupInfoVO queryById(List<GroupTreeVO> groupTreeVOList, GroupInfoVO groupInfoVO, String id) {
        if (CollectionUtils.isEmpty(groupTreeVOList)) {
            return null;
        }
        for (GroupTreeVO groupTreeVO : groupTreeVOList) {
            if (StringUtils.isNotEmpty(groupInfoVO.getId())) {
                return groupInfoVO;
            }
            if (Objects.equals(groupTreeVO.getId(), id)) {
                BeanUtils.copyProperties(groupTreeVO, groupInfoVO);
                return groupInfoVO;
            }
            if (!CollectionUtils.isEmpty(groupTreeVO.getChildren())) {
                queryById(groupTreeVO.getChildren(), groupInfoVO, id);
            }
        }
        return groupInfoVO;
    }

    @Override
    public List<GroupInfoVO> queryAllLeafGroups() {
        // 遍历分组树，获取最末级节点
        List<GroupTreeVO> groupTree = getGroupTree();

        List<GroupInfoVO> result = new ArrayList<>();
        cascadeGetLeafGroup(groupTree, result);
        return result;
    }

    /**
     * 递归查询分组树的叶子节点
     *
     * @param groupTree 分组树节点列表
     * @param result    保存叶子节点的结果集
     * @author lixing
     * @version V1.0 2021/9/19 1:37 下午
     */
    private void cascadeGetLeafGroup(List<GroupTreeVO> groupTree, List<GroupInfoVO> result) {
        if (CollectionUtils.isEmpty(groupTree)) {
            return;
        }
        for (GroupTreeVO groupTreeVO : groupTree) {
            if (CollectionUtils.isEmpty(groupTreeVO.getChildren())) {
                result.add(ConvertGroupTool.INSTANCE.convert2DetailDTO(groupTreeVO));
            } else {
                cascadeGetLeafGroup(groupTreeVO.getChildren(), result);
            }
        }
    }
}
